﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Lesson15
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        class Article
        {
            internal string Title { get; set; }
            internal string Uri { get; set; }
            internal string Date { get; set; }
            internal string Summary { get; set; }
            public override string ToString()
            {
                return Title;
            }
        }

        Dictionary<string, List<Article>> articles = new Dictionary<string, List<Article>>();
        static readonly string[] languages = 
        {
            "C#", "JavaScript", "VB.NET", "Ruby"
        };
        const string CodeZineTop = "http://codezine.jp/";
        const string HrefStart = @"href=""";
        const string HttpUriStart = "http://";
        const string CodeZineUri = "http://codezine.jp";

        string ReadPage(WebClient client, string uri)
        {
            try
            {
                using (var text = new StreamReader(client.OpenRead(uri)))
                {
                    return text.ReadToEnd();
                }
            }
            catch (WebException e)
            {
                return e.Message;
            }
        }

        string CreateUri(string path)
        {
            if (path.IndexOf(HttpUriStart) == 0) return path;
            return ((path[0] == '/') ? CodeZineUri : CodeZineTop) + path;
        }

        string GetLanguageUri(string html, string language)
        {
            var anchorTextPosition = html.IndexOf(language + "</a>");
            if (anchorTextPosition < 0) return string.Empty;
            var hrefPosition = html.LastIndexOf(HrefStart, anchorTextPosition);
            if (hrefPosition < 0) return string.Empty;
            hrefPosition += HrefStart.Length;
            var uriEndPosition = html.IndexOf('"', hrefPosition);
            if (uriEndPosition < 0) return string.Empty;
            var uri = html.Substring(hrefPosition, uriEndPosition - hrefPosition);
            return CreateUri(uri);
        }

        List<Article> CollectArticles(string html)
        {
            var list = new List<Article>();
            var regex = new Regex(@"""day"">([^<]+).+?<a\s+href=""(/article/detail/[^""]+)"">([^<]+)</a>[^p]+p>\s*([^<]+)</p", RegexOptions.Singleline);
            try
            {
                foreach (Match match in regex.Matches(html))
                {
                    list.Add(new Article
                    {
                        Uri = match.Groups[2].Value,
                        Title = match.Groups[3].Value,
                        Date = match.Groups[1].Value,
                        Summary = match.Groups[4].Value
                    });
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            return list;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            using (var client = new WebClient())
            {
                client.Headers.Add(HttpRequestHeader.UserAgent, "MyUserAgent");
                var toppage = ReadPage(client, CodeZineTop);
                foreach (var language in languages)
                {
                    var uri = GetLanguageUri(toppage, language);
                    articles[language] = CollectArticles(ReadPage(client, uri));
                }
            }
            comboBox1.Items.AddRange(languages);
            comboBox1.SelectedIndexChanged += (o, arg) =>
            {
                listBox1.Items.Clear();
                listBox1.Items.AddRange(articles[comboBox1.SelectedItem.ToString()].ToArray());
                if (listBox1.Items.Count > 0)
                {
                    listBox1.SelectedIndex = 0;
                }
                else
                {
                    label1.Text = string.Empty;
                    linkLabel1.Text = string.Empty;
                    textBox1.Text = string.Empty;
                }
            };
            listBox1.SelectedIndexChanged += (o, arg) =>
            {
                var article = listBox1.SelectedItem as Article;
                label1.Text = article.Date;
                linkLabel1.Text = CreateUri(article.Uri);
                linkLabel1.Links[0].LinkData = linkLabel1.Text;
                textBox1.Text = article.Summary;
            };
            linkLabel1.LinkClicked += (o, arg) =>
            {
                Process.Start(arg.Link.LinkData as string);
            };
            comboBox1.SelectedIndex = 0;
        }
    }
}
