﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Lesson16
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        class Article
        {
            internal string Title { get; set; }
            string uri;
            internal string Uri {
                get
                {
                    return (uri.Length > 0) ? Form1.CreateUri(uri) : uri;
                }
                set
                {
                    uri = value;
                }
            }
            internal string Date { get; set; }
            internal string Summary { get; set; }
            public override string ToString()
            {
                return Title;
            }
        }
        static readonly Article NullArticle = new Article
        {
            Title = string.Empty, Uri = string.Empty, Date = string.Empty, Summary = string.Empty
        };
        Dictionary<string, List<Article>> articles = new Dictionary<string, List<Article>>();
        static readonly string[] languages = 
        {
            "C#", "JavaScript", "VB.NET", "Ruby"
        };
        const string CodeZineTop = "http://codezine.jp/";
        const string HrefStart = @"href=""";
        const string HttpUriStart = "http://";
        const string CodeZineUri = "http://codezine.jp";
        const string SelectAll = "すべて";

        string ReadPage(WebClient client, string uri)
        {
            try
            {
                using (var text = new StreamReader(client.OpenRead(uri)))
                {
                    return text.ReadToEnd();
                }
            }
            catch (WebException e)
            {
                return e.Message;
            }
        }

        static string CreateUri(string path)
        {
            if (path.IndexOf(HttpUriStart) == 0) return path;
            return ((path[0] == '/') ? CodeZineUri : CodeZineTop) + path;
        }

        string GetLanguageUri(string html, string language)
        {
            var anchorTextPosition = html.IndexOf(language + "</a>");
            if (anchorTextPosition < 0) return string.Empty;
            var hrefPosition = html.LastIndexOf(HrefStart, anchorTextPosition);
            if (hrefPosition < 0) return string.Empty;
            hrefPosition += HrefStart.Length;
            var uriEndPosition = html.IndexOf('"', hrefPosition);
            if (uriEndPosition < 0) return string.Empty;
            var uri = html.Substring(hrefPosition, uriEndPosition - hrefPosition);
            return CreateUri(uri);
        }

        List<Article> CollectArticles(string html)
        {
            var list = new List<Article>();
            var regex = new Regex(@"""day"">([^<]+).+?<a\s+href=""(/article/detail/[^""]+)"">([^<]+)</a>[^p]+p>\s*([^<]+)</p", RegexOptions.Singleline);
            try
            {
                foreach (Match match in regex.Matches(html))
                {
                    list.Add(new Article
                    {
                        Uri = match.Groups[2].Value,
                        Title = match.Groups[3].Value,
                        Date = match.Groups[1].Value,
                        Summary = match.Groups[4].Value
                    });
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            return list;
        }

        List<Article> CurrentArticles
        {
            get { return articles[comboBox1.SelectedItem as string]; }
        }

        void ShowArticle(Article article)
        {
            label1.Text = article.Date;
            linkLabel1.Text = article.Uri;
            linkLabel1.Links[0].LinkData = linkLabel1.Text;
            textBox1.Text = article.Summary;
        }

        void SetupCurrentTitles()
        {
            listBox1.Items.Clear();
            var list = from a in CurrentArticles
                       where listDate.SelectedIndex == 0 || a.Date == listDate.SelectedItem as string
                       select a;
            listBox1.Items.AddRange(list.ToArray());
            ShowArticle(NullArticle);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            using (var client = new WebClient())
            {
                client.Headers.Add(HttpRequestHeader.UserAgent, "MyUserAgent");
                var toppage = ReadPage(client, CodeZineTop);
                foreach (var language in languages)
                {
                    var uri = GetLanguageUri(toppage, language);
                    articles[language] = CollectArticles(ReadPage(client, uri));
                }
            }
            comboBox1.Items.AddRange(languages);
            comboBox1.SelectedIndexChanged += (o, arg) =>
            {
                listDate.Items.Clear();
                listDate.Items.Add(SelectAll);
                var list = from a in CurrentArticles
                           orderby a.Date descending
                           select a.Date;
                listDate.Items.AddRange(list.Distinct().ToArray());
                listDate.SelectedIndex = 0;
            };
            listDate.SelectedIndexChanged += (o, arg) =>
            {
                SetupCurrentTitles();
            };
            searchText.TextChanged += (o, arg) =>
            {
                listBox1.Items.Clear();
                listBox1.Items.AddRange(
                    (from a in CurrentArticles
                     where
                        (listDate.SelectedIndex == 0 || a.Date == listDate.SelectedItem as string)
                        && a.Summary.Contains(searchText.Text)
                     select a).ToArray());
            };
            listBox1.SelectedIndexChanged += (o, arg) =>
            {
                ShowArticle(listBox1.SelectedItem as Article);
            };
            linkLabel1.LinkClicked += (o, arg) =>
            {
                Process.Start(arg.Link.LinkData as string);
            };
            comboBox1.SelectedIndex = 0;
        }
    }
}
